@@ -14,6 +14,7 @@ from utils import (
     vocabulary,
 )
 
+
 ###############################################################################
 # common defaults
 ###############################################################################
@@ -91,17 +92,12 @@ factory.doc['props']['required_class_default'] = \
 class was given.
 """
 
+
 ###############################################################################
 # generic
 ###############################################################################
 
-def _value(widget, data):
-    """BBB
-    """
-    logging.warn("Deprecated usage of 'yafowil.common._value', please use "+\
-                 "'yafowil.common.fetch_value' instead.") 
-    return fetch_value(widget, data)   
-    
+ 
 def generic_extractor(widget, data):
     """Extract raw data from request by ``widget.dottedpath``.
     """
@@ -110,6 +106,7 @@ def generic_extractor(widget, data):
         return UNSET
     return data.request[widget.dottedpath]
 
+
 @managedprops('required', 'required_message')
 def generic_required_extractor(widget, data):
     """Validate required. 
@@ -138,8 +135,9 @@ def generic_required_extractor(widget, data):
         raise ExtractionError(required)
     raise ExtractionError(widget.attrs['required_message'])
 
+
 @managedprops('type', 'size', 'disabled', 'autofocus', 'placeholder', 
-              *css_managed_props)
+              'autocomplete', *css_managed_props)
 def input_generic_renderer(widget, data):
     """Generic HTML ``input`` tag render.
     """
@@ -154,6 +152,7 @@ def input_generic_renderer(widget, data):
         'placeholder': widget.attrs.get('placeholder') or None,
         'autofocus': widget.attrs.get('autofocus') and 'autofocus' or None,
         'disabled': bool(widget.attrs.get('disabled')) and 'disabled' or None,
+        'autocomplete': widget.attrs.get('autocomplete'),
     }
     html5required = widget.attrs['html5required']
     if html5required:
@@ -165,6 +164,7 @@ def input_generic_renderer(widget, data):
         input_attrs['step'] = widget.attrs.get('step') or None
     return tag('input', **input_attrs)
 
+
 ###############################################################################
 # text
 ###############################################################################
@@ -192,6 +192,7 @@ factory.doc['props']['text.disabled'] = \
 """Flag  input field is disabled.
 """
 
+
 ###############################################################################
 # hidden
 ###############################################################################
@@ -212,6 +213,7 @@ factory.defaults['hidden.default'] = ''
 
 factory.defaults['hidden.class'] = 'hidden'
 
+
 ###############################################################################
 # proxy 
 ###############################################################################
@@ -235,6 +237,7 @@ def input_proxy_renderer(widget, data):
     }
     return tag('input', **input_attrs)
 
+
 factory.register('proxy', 
                  [generic_extractor], 
                  [input_proxy_renderer])
@@ -244,6 +247,7 @@ factory.doc['widget']['proxy'] = \
 
 factory.defaults['proxy.class'] = None
 
+
 ###############################################################################
 # textarea
 ###############################################################################
@@ -266,7 +270,11 @@ def textarea_renderer(widget, data):
         'autofocus': widget.attrs.get('autofocus') and 'autofocus' or None,
         'required': widget.attrs.get('required') and 'required' or None,        
     }
-    return tag('textarea', fetch_value(widget, data), **area_attrs)
+    value = fetch_value(widget, data)
+    if value is None:
+        value = ''
+    return tag('textarea', value, **area_attrs)
+
 
 factory.register('textarea', 
                  [generic_extractor, generic_required_extractor], 
@@ -296,6 +304,7 @@ factory.doc['props']['textarea.readonly'] = \
 """Flag  textarea is readonly.
 """
 
+
 ###############################################################################
 # password
 ###############################################################################
@@ -321,6 +330,7 @@ def minlength_extractor(widget, data):
             raise ExtractionError(message)
     return val
 
+
 @managedprops('ascii')
 def ascii_extractor(widget, data):
     """Validate if a string is ASCII encoding.
@@ -354,6 +364,7 @@ RE_PASSWORD_ALL = [
     SPECIAL_CHAR_RE]
 PASSWORD_NOCHANGE_VALUE = '_NOCHANGE_'
 
+
 @managedprops('strength', 'weak_password_message')
 def password_extractor(widget, data):
     """Extract and validate password input.
@@ -380,7 +391,7 @@ def password_extractor(widget, data):
     if val is UNSET:
         return val
     required_strength = widget.attrs.get('strength', 0)
-    if not required_strength:
+    if required_strength <= 0:
         return val
     if required_strength > len(RE_PASSWORD_ALL):
         required_strength = len(RE_PASSWORD_ALL)
@@ -392,6 +403,7 @@ def password_extractor(widget, data):
         raise ExtractionError(widget.attrs.get('weak_password_message'))
     return val
 
+
 @managedprops('size', 'disabled', 'placeholder', 'autofocus', 'required',
               *css_managed_props)
 def password_renderer(widget, data):
@@ -423,6 +435,7 @@ def password_renderer(widget, data):
             widget.attrs.get('required') and 'required' or None
     return tag('input', **input_attrs)
 
+
 factory.register('password', 
                  [generic_extractor, generic_required_extractor,
                   minlength_extractor, ascii_extractor, password_extractor],
@@ -468,6 +481,7 @@ factory.doc['props']['password.strength'] = \
 """Message shown if password is not strong enough.
 """
 
+
 ###############################################################################
 # checkbox
 ###############################################################################
@@ -485,6 +499,7 @@ def input_checkbox_extractor(widget, data):
         return data.request.get(widget.dottedpath, '')
     raise ValueError, "Checkbox widget has invalid format '%s' set" % format
 
+
 @managedprops('format', *css_managed_props)
 def input_checkbox_renderer(widget, data):
     tag = data.tag
@@ -507,7 +522,9 @@ def input_checkbox_renderer(widget, data):
         'id': cssid(widget, 'checkboxexists'),    
     }
     exists_marker = tag('input', **input_attrs)
-    return checkbox + exists_marker 
+    return checkbox + exists_marker
+
+
 factory.doc['widget']['checkbox'] = """\
 Single checkbox.
 """
@@ -521,6 +538,7 @@ factory.register('checkbox',
                  [input_checkbox_extractor, generic_required_extractor], 
                  [input_checkbox_renderer])
 
+
 ###############################################################################
 # selection
 ###############################################################################
@@ -529,22 +547,38 @@ factory.register('checkbox',
 def select_extractor(widget, data):
     extracted = generic_extractor(widget, data)
     if extracted is UNSET \
-       and widget.attrs['format'] != 'block' \
        and '%s-exists' % widget.dottedpath in data.request:
         if widget.attrs['multivalued']:
             extracted = []
         else:
             extracted = ''
-    return extracted 
+    if widget.attrs['multivalued'] and isinstance(extracted, basestring):
+        extracted = [extracted]
+    return extracted
+
+
+def select_exists_marker(widget, data):
+    tag = data.tag
+    attrs = {
+        'type': 'hidden',
+        'value':  'exists',
+        'name_': "%s-exists" % widget.dottedpath,
+        'id': cssid(widget, 'exists'),
+    }
+    return tag('input', **attrs)
+
 
 @managedprops('format', 'vocabulary', 'multivalued', *css_managed_props)
 def select_renderer(widget, data):
     tag = data.tag
     value = fetch_value(widget, data)
-    if value is UNSET:
-        value = []
+    # Seem to be never called... keep a while
+    #if value is UNSET:
+    #    value = []
     if isinstance(value, basestring) or not hasattr(value, '__iter__'):
         value = [value]
+    if not widget.attrs['multivalued'] and len(value) > 1:
+        raise ValueError(u"Multiple values for single selection.")
     if widget.attrs['format'] == 'block':
         optiontags = []
         for key, term in vocabulary(widget.attrs.get('vocabulary', [])):
@@ -563,7 +597,16 @@ def select_renderer(widget, data):
             'autofocus': widget.attrs.get('autofocus') and 'autofocus' or None,
             'required': widget.attrs.get('required') and 'required' or None,            
         }
-        return tag('select', *optiontags, **select_attrs)
+        rendered = tag('select', *optiontags, **select_attrs)
+        if widget.attrs['multivalued']:
+            attrs = {
+                'type': 'hidden',
+                'value':  'exists',
+                'name_': "%s-exists" % widget.dottedpath,
+                'id': cssid(widget, 'exists'),
+            }
+            rendered = select_exists_marker(widget, data) + rendered
+        return rendered
     else:
         tags = []
         for key, term in vocabulary(widget.attrs.get('vocabulary', [])):
@@ -581,17 +624,11 @@ def select_renderer(widget, data):
             }
             input = tag('input', **attrs)
             text = tag('span', term)
-            tags.append(tag('div', input, text, 
-                            **{'id': cssid(widget, 'radio', key)}))
-        attrs = {
-            'type': 'hidden',
-            'value':  'exists',
-            'name_': "%s-exists" % widget.dottedpath,
-            'id': cssid(widget, 'exists'),    
-        }
-        exists_marker = tag('input', **attrs)            
-        return exists_marker + u''.join(tags)
-        
+            tags.append(tag('div', input, text,
+                            **{'id': cssid(widget, tagtype, key)}))
+        return select_exists_marker(widget, data) + u''.join(tags)
+
+
 factory.doc['widget']['select'] = """\
 Selection Widget. Single selection as dropdown or radio-buttons. Multiple 
 selection as selection-list or as checkboxes. 
@@ -605,10 +642,11 @@ or a callable which returns one of both first. An iterable can consist out of
 strings or out of tuples with ``(key, value)``.   
 """
 
-factory.register('select', 
-                 [select_extractor], 
+factory.register('select',
+                 [select_extractor, generic_required_extractor], 
                  [select_renderer])
 
+
 ###############################################################################
 # file
 ###############################################################################
@@ -625,7 +663,9 @@ def file_extracor(widget, data):
             return UNSET
     return data.request[name]
 
-@managedprops('accept', 'placeholder', 'autofocus', 'required', *css_managed_props)
+
+@managedprops('accept', 'placeholder', 'autofocus', 
+              'required', *css_managed_props)
 def input_file_renderer(widget, data):
     tag = data.tag
     input_attrs = {
@@ -642,6 +682,7 @@ def input_file_renderer(widget, data):
         input_attrs['accept'] = widget.attrs['accept']
     return tag('input', **input_attrs)
 
+
 @managedprops(*css_managed_props)
 def file_options_renderer(widget, data):
     if data.value in [None, UNSET, '']:
@@ -667,6 +708,7 @@ def file_options_renderer(widget, data):
                         **{'id': cssid(widget, 'radio', key)}))
     return data.rendered + u''.join(tags)
 
+
 factory.doc['widget']['file'] = """\
 A basic file upload widget.
 """
@@ -685,6 +727,7 @@ factory.register('file',
                  [file_extracor, generic_required_extractor],
                  [input_file_renderer, file_options_renderer])
 
+
 ###############################################################################
 # submit
 ###############################################################################
@@ -700,6 +743,8 @@ def submit_renderer(widget, data):
         'value': widget.attrs.get('label', widget.__name__),
     }
     return tag('input', **input_attrs)
+
+
 factory.doc['widget']['submit'] = """\
 Submit tag inside the form
 """
@@ -727,25 +772,34 @@ Next is a callable expected to return the web address. It expects a request as
 the only parameter. 
 """
 
+factory.defaults['text.disabled'] = False
+factory.doc['props']['text.disabled'] = \
+"""Flag  input field is disabled.
+"""
+
 factory.register('submit', [], [submit_renderer])
 
+
 ###############################################################################
 # email
 ###############################################################################
 
 EMAIL_RE = u'^[a-zA-Z0-9\._\-]+@[a-zA-Z0-9\._\-]+.[a-zA-Z0-9]{2,6}$'
 
+
 def email_extractor(widget, data):
     val = data.extracted
     if not re.match(EMAIL_RE, val is not UNSET and val or ''):
         raise ExtractionError(u'Input not a valid email address.')
     return val
 
+
 def email_renderer(widget, data):
     if not widget.attrs['html5type']:
         widget.attrs['type'] = 'text'
     return input_generic_renderer(widget, data)
 
+
 factory.doc['widget']['email'] = \
 """E-mail (HTML5) input widget.
 """
@@ -760,6 +814,7 @@ factory.register('email',
                   email_extractor],
                  [email_renderer])
 
+
 ###############################################################################
 # url
 ###############################################################################
@@ -767,12 +822,14 @@ factory.register('email',
 URL_RE = u'^(ftp|http|https):\/\/(\w+:{0,1}\w*@)?(\S+)(:[0-9]+)?(\/|\/([\w#!:'+\
          u'.?+=&%@!\-\/]))?$'
 
+
 def url_extractor(widget, data):
     val = data.extracted
     if not re.match(URL_RE, val is not UNSET and val or ''):
         raise ExtractionError(u'Input not a valid web address.')
     return val
 
+
 factory.doc['widget']['url'] = \
 """URL aka web address (HTML5) input widget.
 """
@@ -786,6 +843,7 @@ factory.register('url',
                   url_extractor],
                  [input_generic_renderer])
 
+
 ###############################################################################
 # search
 ###############################################################################
@@ -801,12 +859,15 @@ factory.register('search',
                  [generic_extractor, generic_required_extractor],
                  [input_generic_renderer])
 
+
 ###############################################################################
 # number
 ###############################################################################
 
 def number_extractor(widget, data):
     val = data.extracted
+    if val is UNSET:
+        return val
     if widget.attrs.get('datatype') == 'integer':
         convert = int
     elif widget.attrs.get('datatype') == 'float':
@@ -830,6 +891,7 @@ def number_extractor(widget, data):
                               widget.attrs.get('step'))        
     return val
 
+
 factory.doc['widget']['number'] = """\
 Number widget (HTML5).
 """
@@ -851,28 +913,40 @@ factory.register('number',
                   number_extractor],
                  [input_generic_renderer])
 
+
 ###############################################################################
 # label
 ###############################################################################
 
-@managedprops('position', 'label', 'help', 'help_class', *css_managed_props)
+@managedprops('position', 'label', 'for', 'help', 'help_class',
+              *css_managed_props)
 def label_renderer(widget, data):
     tag = data.tag
     label_text = widget.attrs.get('label', widget.__name__)
+    for_path = widget.attrs['for']
+    if for_path:
+        for_widget = widget.root
+        for name in for_path.split('.'):
+            for_widget = for_widget[name]
+        for_ = cssid(for_widget, 'input')
+    else:
+        for_ = cssid(widget, 'input')
     label_attrs = {
-        'for_': cssid(widget, 'input'),
+        'for_': for_,
         'class_': cssclasses(widget, data)
     }
     help = u''
     if widget.attrs['help']:
         help_attrs = {'class_': widget.attrs['help_class']}
-        help = tag('div', widget.attrs['help'], help_attrs)
+        help = tag('div', widget.attrs['help'], **help_attrs)
     pos = widget.attrs['position']
+    rendered = data.rendered is not UNSET and data.rendered or u''
     if pos == 'inner':
-        return tag('label', label_text, help, data.rendered, **label_attrs)
+        return tag('label', label_text, help, rendered, **label_attrs)
     elif pos == 'after':
-        return data.rendered + tag('label', label_text, help, **label_attrs)
-    return tag('label', label_text, help, **label_attrs) + data.rendered
+        return rendered + tag('label', label_text, help, **label_attrs)
+    return tag('label', label_text, help, **label_attrs) + rendered
+
 
 factory.doc['widget']['label'] = """\
 Label widget.
@@ -887,15 +961,20 @@ the label tag.
 factory.doc['props']['label.label'] = """\
 Text to be displayed as a label.
 """
+factory.doc['props']['label.for'] = """\
+Optional dottedpath of widget to be labled
+"""
 factory.doc['props']['label.help'] = """\
 Optional help text (alternative description) to be rendered inside a div after
 the label.
 """
 
+factory.defaults['label.for'] = None
 factory.defaults['label.help'] = None
 factory.defaults['label.help_class'] = 'help'
 factory.register('label', [], [label_renderer])
 
+
 ###############################################################################
 # field
 ###############################################################################
@@ -911,6 +990,7 @@ def field_renderer(widget, data):
         div_attrs['class_'] += u' %s' % widget.attrs['witherror']
     return tag('div', data.rendered, **div_attrs)
 
+
 factory.doc['widget']['field'] = """\
 Renders a div with an class field around the prior rendered output. This is 
 supposed to be used for styling and grouping purposes.
@@ -922,6 +1002,7 @@ Put the class given with this property on the div if an error happened.
 """
 factory.register('field', [], [field_renderer])
 
+
 ###############################################################################
 # error
 ###############################################################################
@@ -936,9 +1017,10 @@ def error_renderer(widget, data):
         msgs += tag('div', str(error), class_=widget.attrs['message_class'])
     return tag('div', msgs, data.rendered, class_=cssclasses(widget, data))
 
+
 factory.doc['widget']['error'] = """\
 Renders a div with an errormessage around the prior rendered output.
 """
 factory.defaults['error.error_class'] = 'error'
 factory.defaults['error.message_class'] = 'errormessage'
-factory.register('error', [], [error_renderer])
+factory.register('error', [], [error_renderer])
\ No newline at end of file
